# Copyright (c) 2017 VMware, Inc.  All rights reserved.
from bag import Bag
from ipaddr import IpAddr
from operator import or_
from version import Version
import re

class InterfaceFactory:
    @staticmethod
    def create(bag, offset):
        v = Version().get_version()
        
        if v[0] == 'RedHat':
            return RedHatInterface(bag, offset)
        elif v[0] == 'SuSE':
            return SuseInterface(bag, offset)

class Interface(object):
    def __init__(self, bag, offset):
        self.bag = bag
        self.offset = offset

    def default(self):
        return "VirtualMachine.Network.Default"

    def is_default(self):
        return self.bag.has_key(self.default())

    def address(self):
        return "VirtualMachine.Network%s.Address" % self.offset

    def is_address(self):
        return self.bag.has_key(self.address())

    def subnet_mask(self):
        return "VirtualMachine.Network%s.SubnetMask" % self.offset

    def is_subnet_mask(self):
        return self.bag.has_key(self.subnet_mask())

    def mac(self):
        return "VirtualMachine.Network%s.MacAddress" % self.offset

    def is_mac(self):
        return self.bag.has_key(self.mac())

    def gateway(self):
        return 'VirtualMachine.Network%s.Gateway' % self.offset

    def is_gateway(self):
        return self.bag.has_key(self.gateway())

#     def ignores(self):
#     def lines_dhcp(self):
#     def lines_static(self):

    def lines(self, ifcfg_eth_lines):
        lines = []
        for line in ifcfg_eth_lines:
            if not reduce(or_, map(lambda x: line.startswith(x), self.ignores())):
                lines.append(line.strip())

        if self.is_address():
            return self.lines_static(lines)

        return lines

class RedHatInterface(Interface):
    def lines_dhcp(self, lines):
        lines.append("BOOTPROTO=dhcp")
        lines.append("DEVICE=eth%s" % self.offset)
        lines.append("ONBOOT=yes")

        return lines

    def lines_static(self, lines):
        addr = self.bag[self.address()]
        mask = self.bag[self.subnet_mask()]
        ipv4 = IpAddr(addr, mask)

        lines.append("BOOTPROTO=static")
        lines.append("BROADCAST=%s" % ipv4.broadcast())
        lines.append("DEVICE=eth%s" % self.offset)
        lines.append("IPADDR=%s" % addr)
        lines.append("ONBOOT=yes")
        lines.append("NETMASK=%s" % mask)
        lines.append("NETWORK=%s" % ipv4.network())

        return lines

    def ignores(self):
        return ('BOOTPROTO=', 'BROADCAST=', 'DEVICE=', 'IPADDR=', 'ONBOOT=', 'NETMASK=', 'NETWORK')


class SuseInterface(Interface):
    def lines_dhcp(self, lines):
        lines.append('BOOTPROTO=dhcp')
        lines.append('STARTMODE=auto')
        
        return lines

    def lines_static(self, lines):
        addr = self.bag[self.address()]
        mask = self.bag[self.subnet_mask()]
        ipv4 = IpAddr(addr, mask)

        lines.append("BOOTPROTO=static")
        lines.append("BROADCAST=%s" % ipv4.broadcast())
        lines.append("IPADDR=%s" % addr)
        lines.append("NETMASK=%s" % mask)
        lines.append("STARTMODE=auto")

        return lines

    def ignores(self):
        return ('BOOTPROTO=', 'BROADCAST=', 'IPADDR=', 'STARTMODE=', 'NETMASK=')

class UbuntuInterface(Interface):
    def get_first_index_of_iface_stanza(self, lines):
        try:
            return lines.index(next(x for x in lines if x.startswith('iface eth%s inet' % self.offset)))
        except StopIteration:
            return len(lines) - 1

    def get_last_index_of_stanza(self, lines, first_index):
        pattern = re.compile('\A(auto|allow-auto|allow-hotplug|iface|mapping) .+')
        
        try:
            return lines.index(next(x for x in lines[first_index:] if pattern.match(x))) + first_index - 1
        except StopIteration:
            return len(lines) - 1

    def lines(self, interfaces_lines):
        interfaces_lines_str = [x.strip(' \t\n') for x in interfaces_lines]
        
        if 'auto eth%s' % self.offset in interfaces_lines_str:
            interfaces_lines_str.remove('auto eth%s' % self.offset)
        
        first = self.get_first_index_of_iface_stanza(interfaces_lines_str)
        last = self.get_last_index_of_stanza(interfaces_lines_str, first)
        stanza_lines = interfaces_lines_str[first:last + 1]
        new_stanza_lines = super(UbuntuInterface, self).lines(stanza_lines)
	
        result = []
        result.extend(interfaces_lines_str[0:first])
        result.extend(new_stanza_lines)
        result.extend(interfaces_lines_str[last + 1:len(interfaces_lines_str)])
	
        return result

    #def lines_dhcp(self, lines):

    def lines_static(self, lines):
        addr = self.bag[self.address()]
        mask = self.bag[self.subnet_mask()]
        ipv4 = IpAddr(addr, mask)

        lines.append('auto eth%s' % self.offset)
        lines.append('iface eth%s inet static' % self.offset)
        lines.append('address %s' % addr)
        lines.append('network %s' % ipv4.network())
        lines.append('netmask %s' % mask)
        lines.append('broadcast %s' % ipv4.broadcast())

        default_offset = 0
        if self.is_default():
            default_offset = self.bag[self.default()]

        if self.is_gateway() and self.offset == default_offset:
            lines.append('gateway %s' % self.bag[self.gateway()])
        
        return lines

    def ignores(self):
        return ('dns-search', 'dns-nameservers', 'auto eth%s' % self.offset, 'iface eth%s inet' % self.offset, 'address', 'network', 'netmask', 'broadcast', 'gateway')

# vim: sw=4 ts=4 et
